@@ -7,6 +7,7 @@ use ast_grep_language::Language;
 use clap::{builder::PossibleValue, Parser, ValueEnum};
 use ignore::WalkParallel;
 
+use crate::config::ProjectConfig;
 use crate::lang::SgLang;
 use crate::print::{ColoredPrinter, Diff, Heading, InteractivePrinter, JSONPrinter, Printer};
 use crate::utils::ErrorContext as EC;
@@ -140,7 +141,9 @@ impl RunArg {
 
 // Every run will include Search or Replace
 // Search or Replace by arguments `pattern` and `rewrite` passed from CLI
-pub fn run_with_pattern(arg: RunArg) -> Result<()> {
+pub fn run_with_pattern(arg: RunArg, project: Result<ProjectConfig>) -> Result<()> {
+  let proj = arg.output.inspect.project_trace();
+  proj.print_project(&project)?;
   let context = arg.context.get();
   if let Some(json) = arg.output.json {
     let printer = JSONPrinter::stdout(json).context(context);
@@ -160,12 +163,12 @@ pub fn run_with_pattern(arg: RunArg) -> Result<()> {
 }
 
 fn run_pattern_with_printer(arg: RunArg, printer: impl Printer + 'static) -> Result<()> {
+  let trace = arg.output.inspect.run_trace();
   if arg.input.stdin {
-    RunWithSpecificLang::new(arg)?.run_std_in(printer)
+    RunWithSpecificLang::new(arg, trace)?.run_std_in(printer)
   } else if arg.lang.is_some() {
-    RunWithSpecificLang::new(arg)?.run_path(printer)
+    RunWithSpecificLang::new(arg, trace)?.run_path(printer)
   } else {
-    let trace = arg.output.inspect.run_trace();
     RunWithInferredLang { arg, trace }.run_path(printer)
   }
 }
@@ -206,7 +209,7 @@ impl PathWorker for RunWithInferredLang {
     self.arg.input.walk()
   }
   fn get_trace(&self) -> &FileTrace {
-    &self.trace.file_trace
+    &self.trace.inner
   }
 
   fn produce_item(&self, path: &Path) -> Option<Vec<Self::Item>> {
@@ -234,7 +237,7 @@ struct RunWithSpecificLang {
 }
 
 impl RunWithSpecificLang {
-  fn new(arg: RunArg) -> Result<Self> {
+  fn new(arg: RunArg, stats: RunTrace) -> Result<Self> {
     let lang = arg.lang.ok_or(anyhow::anyhow!(EC::LanguageNotSpecified))?;
     // do not unwrap result here
     let pattern_ret = arg.build_pattern(lang);
@@ -244,7 +247,6 @@ impl RunWithSpecificLang {
     } else {
       None
     };
-    let stats = arg.output.inspect.run_trace();
     Ok(Self {
       arg,
       pattern: pattern_ret?,
@@ -280,7 +282,7 @@ impl PathWorker for RunWithSpecificLang {
     Ok(self.arg.input.walk_lang(lang))
   }
   fn get_trace(&self) -> &FileTrace {
-    &self.stats.file_trace
+    &self.stats.inner
   }
   fn produce_item(&self, path: &Path) -> Option<Vec<Self::Item>> {
     let arg = &self.arg;
@@ -374,7 +376,8 @@ mod test {
       pattern: "console.log".to_string(),
       ..default_run_arg()
     };
-    assert!(run_with_pattern(arg).is_ok())
+    let proj = Err(anyhow::anyhow!("no project"));
+    assert!(run_with_pattern(arg, proj).is_ok())
   }
 
   #[test]
@@ -384,7 +387,8 @@ mod test {
       strictness: Some(Strictness(MatchStrictness::Ast)),
       ..default_run_arg()
     };
-    assert!(run_with_pattern(arg).is_ok())
+    let proj = Err(anyhow::anyhow!("no project"));
+    assert!(run_with_pattern(arg, proj).is_ok())
   }
 
   #[test]
@@ -394,6 +398,7 @@ mod test {
       lang: Some(SupportLang::Rust.into()),
       ..default_run_arg()
     };
-    assert!(run_with_pattern(arg).is_ok())
+    let proj = Err(anyhow::anyhow!("no project"));
+    assert!(run_with_pattern(arg, proj).is_ok())
   }
 }
