@@ -73,6 +73,8 @@ impl ScanArg {
 }
 
 pub fn run_with_config(arg: ScanArg, project: Result<ProjectConfig>) -> Result<()> {
+  let project_trace = arg.output.inspect.project_trace();
+  project_trace.print_project(&project)?;
   let context = arg.context.get();
   if let Some(_format) = &arg.format {
     let printer = CloudPrinter::stdout();
@@ -198,7 +200,7 @@ fn unused_suppression_rule_config(arg: &ScanArg, overwrite: &RuleOverwrite) -> R
 
 impl PathWorker for ScanWithConfig {
   fn get_trace(&self) -> &FileTrace {
-    &self.trace.file_trace
+    &self.trace.inner.file_trace
   }
   fn build_walk(&self) -> Result<WalkParallel> {
     let mut langs = HashSet::new();
