@@ -6,11 +6,11 @@ use tower_lsp::jsonrpc::Result;
 use tower_lsp::lsp_types::*;
 use tower_lsp::{Client, LanguageServer};
 
-use ast_grep_config::{CombinedScan, RuleCollection, RuleConfig};
+use ast_grep_config::{CombinedScan, RuleCollection, RuleConfig, Severity};
 use ast_grep_core::{language::Language, AstGrep, Doc, StrDoc};
 
 use std::collections::HashMap;
-use std::path::{Path, PathBuf};
+use std::path::PathBuf;
 
 use utils::{convert_match_to_diagnostic, diagnostic_to_code_action, RewriteData};
 
@@ -186,14 +186,12 @@ impl<L: LSPLang> Backend<L> {
 
   fn get_rules(&self, uri: &Url) -> Option<Vec<&RuleConfig<L>>> {
     let absolute_path = uri.to_file_path().ok()?;
-    // for_path needs relative path, see https://github.com/ast-grep/ast-grep/issues/1272
-    let base = Path::new("./");
     let path = if let Ok(p) = absolute_path.strip_prefix(&self.base) {
-      base.join(p)
+      p
     } else {
-      absolute_path
+      &absolute_path
     };
-    let rules = self.rules.as_ref().ok()?.for_path(&path);
+    let rules = self.rules.as_ref().ok()?.for_path(path);
     Some(rules)
   }
 
@@ -203,12 +201,17 @@ impl<L: LSPLang> Backend<L> {
     versioned: &VersionedAst<StrDoc<L>>,
   ) -> Option<Vec<Diagnostic>> {
     let rules = self.get_rules(uri)?;
-    let scan = CombinedScan::new(rules);
+    if rules.is_empty() {
+      return None;
+    }
+    let unused_suppression_rule =
+      CombinedScan::unused_config(Severity::Hint, rules[0].language.clone());
+    let mut scan = CombinedScan::new(rules);
+    scan.set_unused_suppression_rule(&unused_suppression_rule);
     let pre_scan = scan.find(&versioned.root);
     let matches = scan.scan(&versioned.root, pre_scan, false).matches;
     let mut diagnostics = vec![];
-    for (id, ms) in matches {
-      let rule = scan.get_rule(id);
+    for (rule, ms) in matches {
       let to_diagnostic = |m| convert_match_to_diagnostic(m, rule);
       diagnostics.extend(ms.into_iter().map(to_diagnostic));
     }
